'use strict';

var Data = require('effect/Data');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var Data__namespace = /*#__PURE__*/_interopNamespace(Data);

class BadRequestStatusError extends Data__namespace.TaggedError(
  "BadRequestStatusError"
) {
  static status = 400;
  constructor() {
    super({
      message: "The server will not process the request.",
      status: BadRequestStatusError.status
    });
  }
}
class PaymentRequiredStatusError extends Data__namespace.TaggedError(
  "PaymentRequiredStatusError"
) {
  static status = 402;
  constructor() {
    super({
      message: "The shop is frozen. The shop owner will need to pay the outstanding balance to unfreeze the shop.",
      status: PaymentRequiredStatusError.status
    });
  }
}
class ForbiddenStatusError extends Data__namespace.TaggedError(
  "ForbiddenStatusError"
) {
  static status = 403;
  constructor() {
    super({
      message: "The shop is forbidden. Returned if the store has been marked as fraudulent.",
      status: ForbiddenStatusError.status
    });
  }
}
class NotFoundStatusError extends Data__namespace.TaggedError(
  "NotFoundStatusError"
) {
  static status = 404;
  constructor() {
    super({
      message: "The resource isn\u2019t available. This is often caused by querying for something that\u2019s been deleted.",
      status: NotFoundStatusError.status
    });
  }
}
class LockedStatusError extends Data__namespace.TaggedError("LockedStatusError") {
  static status = 423;
  constructor() {
    super({
      status: LockedStatusError.status,
      message: "The shop isn\u2019t available. This can happen when stores repeatedly exceed API rate limits or due to fraud risk."
    });
  }
}
class RetriableStatusCodesError extends Data__namespace.TaggedError(
  "RetriableStatusCodesError"
) {
  static validStatuses = [429, 503];
  constructor(status) {
    super({
      message: "Something went wrong with the request. Attempted to retry the request, but failed.",
      status
    });
  }
}
class StorefrontServerStatusError extends Data__namespace.TaggedError(
  "StorefrontServerStatusError"
) {
  constructor(status) {
    super({
      message: "An internal error occurred in Shopify. Check out the Shopify status page for more information.",
      status
    });
  }
}
class AssertQueryError extends Data__namespace.TaggedError("AssertQueryError") {
  constructor(query) {
    super({
      message: "Invalid query string provided!",
      query
    });
  }
}
class AssertMutationError extends Data__namespace.TaggedError(
  "AssertMutationError"
) {
  constructor(mutation) {
    super({
      message: "Invalid mutation string provided!",
      mutation
    });
  }
}
class ExtractOperationNameError extends Data__namespace.TaggedError(
  "ExtractOperationNameError"
) {
  constructor(operation) {
    super({
      message: `Failed to extract operation name from operation: ${operation}`
    });
  }
}

exports.AssertMutationError = AssertMutationError;
exports.AssertQueryError = AssertQueryError;
exports.BadRequestStatusError = BadRequestStatusError;
exports.ExtractOperationNameError = ExtractOperationNameError;
exports.ForbiddenStatusError = ForbiddenStatusError;
exports.LockedStatusError = LockedStatusError;
exports.NotFoundStatusError = NotFoundStatusError;
exports.PaymentRequiredStatusError = PaymentRequiredStatusError;
exports.RetriableStatusCodesError = RetriableStatusCodesError;
exports.StorefrontServerStatusError = StorefrontServerStatusError;
//# sourceMappingURL=errors.cjs.map
//# sourceMappingURL=errors.cjs.map