'use strict';

var Effect = require('effect/Effect');
var TypedStorefrontClient = require('./services/TypedStorefrontClient.cjs');
var Logger = require('effect/Logger');
var LogLevel = require('effect/LogLevel');
var Layer = require('effect/Layer');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var Effect__namespace = /*#__PURE__*/_interopNamespace(Effect);
var TypedStorefrontClient__namespace = /*#__PURE__*/_interopNamespace(TypedStorefrontClient);
var Logger__namespace = /*#__PURE__*/_interopNamespace(Logger);
var LogLevel__namespace = /*#__PURE__*/_interopNamespace(LogLevel);
var Layer__namespace = /*#__PURE__*/_interopNamespace(Layer);

const createStorefrontClient = ({
  logger,
  ...initOptions
}) => {
  let layer = TypedStorefrontClient__namespace.Default;
  let minimumLogLevel = LogLevel__namespace.None;
  let loggerLayer = Logger__namespace.pretty;
  if (logger) {
    loggerLayer = Logger__namespace.replace(Logger__namespace.defaultLogger, Logger__namespace.make(logger));
    layer = Layer__namespace.mergeAll(loggerLayer, layer);
    minimumLogLevel = LogLevel__namespace.All;
  }
  return Effect__namespace.runSync(
    Effect__namespace.gen(function* () {
      const client = yield* TypedStorefrontClient__namespace.make(initOptions);
      return {
        mutate: (mutation, options) => Effect__namespace.runPromise(
          client.mutate(mutation, options).pipe(
            Logger__namespace.withMinimumLogLevel(minimumLogLevel),
            Effect__namespace.provide(loggerLayer),
            Effect__namespace.catchAll((error) => {
              if (error._tag === "ExtractOperationNameError" || error._tag === "ParseError" || error._tag === "RequestError" || error._tag === "ResponseError") {
                return Effect__namespace.fail(
                  new Error(error.message, {
                    cause: error.cause
                  })
                );
              }
              if (error._tag === "HttpBodyError") {
                const reason = error.reason;
                if (reason._tag === "JsonError") {
                  return Effect__namespace.fail(
                    new Error("Failed to parse JSON response")
                  );
                }
                if (reason._tag === "SchemaError") {
                  return Effect__namespace.fail(
                    new Error(reason.error.message, {
                      cause: reason.error.cause
                    })
                  );
                }
              }
              return Effect__namespace.fail(new Error("Something went horribly wrong!"));
            })
          ),
          {
            signal: options?.signal
          }
        ),
        query: (query, options) => Effect__namespace.runPromise(
          client.query(query, options).pipe(
            Logger__namespace.withMinimumLogLevel(minimumLogLevel),
            Effect__namespace.provide(loggerLayer),
            Effect__namespace.catchAll((error) => {
              if (error._tag === "ExtractOperationNameError" || error._tag === "ParseError" || error._tag === "RequestError" || error._tag === "ResponseError") {
                return Effect__namespace.fail(
                  new Error(error.message, {
                    cause: error.cause
                  })
                );
              }
              if (error._tag === "HttpBodyError") {
                const reason = error.reason;
                if (reason._tag === "JsonError") {
                  return Effect__namespace.fail(
                    new Error("Failed to parse JSON response")
                  );
                }
                if (reason._tag === "SchemaError") {
                  return Effect__namespace.fail(
                    new Error(reason.error.message, {
                      cause: reason.error.cause
                    })
                  );
                }
              }
              return Effect__namespace.fail(new Error("Something went horribly wrong!"));
            })
          ),
          {
            signal: options?.signal
          }
        )
      };
    }).pipe(Logger__namespace.withMinimumLogLevel(minimumLogLevel), Effect__namespace.provide(layer))
  );
};

exports.createStorefrontClient = createStorefrontClient;
//# sourceMappingURL=index.cjs.map
//# sourceMappingURL=index.cjs.map