'use strict';

var FetchHttpClient = require('@effect/platform/FetchHttpClient');
var HttpClient = require('@effect/platform/HttpClient');
var HttpClientResponse = require('@effect/platform/HttpClientResponse');
var HttpClientRequest = require('@effect/platform/HttpClientRequest');
var DefaultHeaders = require('./DefaultHeaders.cjs');
var PublicHeaders = require('./PublicHeaders.cjs');
var PrivateHeaders = require('./PrivateHeaders.cjs');
var ResponseErrors = require('../data/ResponseErrors.cjs');
var ClientResponse = require('../data/ClientResponse.cjs');
var Context = require('effect/Context');
var Effect = require('effect/Effect');
var Layer = require('effect/Layer');
var Redacted = require('effect/Redacted');
var Schema = require('effect/Schema');
var Schedule = require('effect/Schedule');
var Function = require('effect/Function');
var storefront_js = require('../utils/storefront.cjs');
var schemas_js = require('../schemas.cjs');
var constants_js = require('../constants.cjs');
var errors_js = require('../errors.cjs');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var FetchHttpClient__namespace = /*#__PURE__*/_interopNamespace(FetchHttpClient);
var HttpClient__namespace = /*#__PURE__*/_interopNamespace(HttpClient);
var HttpClientResponse__namespace = /*#__PURE__*/_interopNamespace(HttpClientResponse);
var HttpClientRequest__namespace = /*#__PURE__*/_interopNamespace(HttpClientRequest);
var DefaultHeaders__namespace = /*#__PURE__*/_interopNamespace(DefaultHeaders);
var PublicHeaders__namespace = /*#__PURE__*/_interopNamespace(PublicHeaders);
var PrivateHeaders__namespace = /*#__PURE__*/_interopNamespace(PrivateHeaders);
var ResponseErrors__namespace = /*#__PURE__*/_interopNamespace(ResponseErrors);
var ClientResponse__namespace = /*#__PURE__*/_interopNamespace(ClientResponse);
var Context__namespace = /*#__PURE__*/_interopNamespace(Context);
var Effect__namespace = /*#__PURE__*/_interopNamespace(Effect);
var Layer__namespace = /*#__PURE__*/_interopNamespace(Layer);
var Redacted__namespace = /*#__PURE__*/_interopNamespace(Redacted);
var Schema__namespace = /*#__PURE__*/_interopNamespace(Schema);
var Schedule__namespace = /*#__PURE__*/_interopNamespace(Schedule);
var Function__namespace = /*#__PURE__*/_interopNamespace(Function);

const make = (initOptions) => Effect__namespace.gen(function* () {
  const defaultOptions = yield* Schema__namespace.decode(schemas_js.ClientOptions)(initOptions);
  const defaultClient = yield* HttpClient__namespace.HttpClient;
  const defaultEndpoint = storefront_js.buildStorefrontApiUrl({
    apiVersion: defaultOptions.apiVersion,
    storeName: defaultOptions.storeName
  });
  const client = defaultClient.pipe(
    HttpClient__namespace.mapRequestInput(
      (request) => HttpClientRequest__namespace.setHeaders(request, {
        ...DefaultHeaders__namespace.makeFallback(request, {
          contentType: defaultOptions.contentType,
          apiVersion: defaultOptions.apiVersion
        }),
        ...PublicHeaders__namespace.makeFallback(request, {
          publicAccessToken: defaultOptions.publicAccessToken
        }),
        ...PrivateHeaders__namespace.makeFallback(request, {
          privateAccessToken: defaultOptions.privateAccessToken ? Redacted__namespace.value(defaultOptions.privateAccessToken) : void 0
        })
      })
    ),
    HttpClient__namespace.transformResponse(
      (response) => response.pipe(
        Effect__namespace.filterOrFail(
          (res) => res.status !== errors_js.BadRequestStatusError.status,
          () => new errors_js.BadRequestStatusError()
        ),
        Effect__namespace.filterOrFail(
          (res) => res.status !== errors_js.PaymentRequiredStatusError.status,
          () => new errors_js.PaymentRequiredStatusError()
        ),
        Effect__namespace.filterOrFail(
          (res) => res.status !== errors_js.ForbiddenStatusError.status,
          () => new errors_js.ForbiddenStatusError()
        ),
        Effect__namespace.filterOrFail(
          (res) => res.status !== errors_js.NotFoundStatusError.status,
          () => new errors_js.NotFoundStatusError()
        ),
        Effect__namespace.filterOrFail(
          (res) => res.status !== errors_js.LockedStatusError.status,
          () => new errors_js.LockedStatusError()
        ),
        Effect__namespace.filterOrElse(
          (res) => !errors_js.RetriableStatusCodesError.validStatuses.includes(res.status),
          (res) => new errors_js.RetriableStatusCodesError(res.status)
        ),
        Effect__namespace.filterOrElse(
          (res) => res.status < 500,
          (res) => new errors_js.StorefrontServerStatusError(res.status)
        )
      )
    ),
    HttpClient__namespace.retry({
      times: defaultOptions.retries,
      schedule: Schedule__namespace.spaced(`${constants_js.RETRY_WAIT_TIME} millis`),
      while: (error) => {
        if (error._tag === "RetriableStatusCodesError") return true;
        return false;
      }
    })
  );
  const makeRequest = (operation, options) => Effect__namespace.gen(function* () {
    const validatedOptions = yield* Schema__namespace.decodeUnknown(schemas_js.RequestOptions)(
      options || {}
    );
    let endpoint = defaultEndpoint;
    if (options?.apiVersion || options?.storeName)
      endpoint = storefront_js.buildStorefrontApiUrl({
        apiVersion: validatedOptions.apiVersion || defaultOptions.apiVersion,
        storeName: validatedOptions.storeName || defaultOptions.storeName
      });
    const request = HttpClientRequest__namespace.post(endpoint).pipe(
      HttpClientRequest__namespace.setHeaders({
        ...DefaultHeaders__namespace.make({
          contentType: validatedOptions?.contentType,
          apiVersion: validatedOptions?.apiVersion
        }),
        ...PublicHeaders__namespace.make({
          publicAccessToken: validatedOptions?.publicAccessToken
        }),
        ...PrivateHeaders__namespace.make({
          privateAccessToken: validatedOptions?.privateAccessToken ? Redacted__namespace.value(validatedOptions?.privateAccessToken) : void 0,
          buyerIp: validatedOptions?.buyerIp
        })
      }),
      HttpClientRequest__namespace.bodyJson({
        query: operation,
        variables: validatedOptions?.variables
      })
    );
    return yield* request;
  });
  const executeRequest = (operation, options) => Effect__namespace.gen(function* () {
    const request = yield* makeRequest(operation, options);
    const response = yield* client.execute(request);
    const json = yield* Function__namespace.pipe(
      response,
      HttpClientResponse__namespace.schemaBodyJson(schemas_js.GraphQLJsonBody)
    );
    if (json.errors) {
      return ClientResponse__namespace.make({
        extensions: json.extensions,
        errors: ResponseErrors__namespace.make({
          networkStatusCode: response.status,
          graphQLErrors: json.errors
        })
      });
    }
    return ClientResponse__namespace.make({
      data: json.data,
      extensions: json.extensions
    });
  }).pipe(
    Effect__namespace.catchAll((error) => {
      if (error._tag === "BadRequestStatusError" || error._tag === "ForbiddenStatusError" || error._tag === "LockedStatusError" || error._tag === "NotFoundStatusError" || error._tag === "PaymentRequiredStatusError" || error._tag === "RetriableStatusCodesError" || error._tag === "StorefrontServerStatusError") {
        return Effect__namespace.succeed(
          ClientResponse__namespace.make({
            errors: ResponseErrors__namespace.make({
              networkStatusCode: error.status,
              graphQLErrors: [],
              message: error.message
            })
          })
        );
      }
      return Effect__namespace.fail(error);
    }),
    Effect__namespace.tapError(Effect__namespace.logError)
  );
  return {
    request: executeRequest
  };
}).pipe(Effect__namespace.tapError(Effect__namespace.logError));
class StorefrontClient extends Context__namespace.Tag(
  "@solidifront/storefront-client/StorefrontClient"
)() {
}
const Default = Layer__namespace.mergeAll(
  Layer__namespace.succeed(StorefrontClient, StorefrontClient.of(make)),
  FetchHttpClient__namespace.layer
);

exports.Default = Default;
exports.StorefrontClient = StorefrontClient;
exports.make = make;
//# sourceMappingURL=StorefrontClient.cjs.map
//# sourceMappingURL=StorefrontClient.cjs.map