'use strict';

var Effect = require('effect/Effect');
var LoggerUtils = require('./LoggerUtils.cjs');
var LogLevel = require('effect/LogLevel');
var predicates_js = require('../predicates.cjs');
var errors_js = require('../errors.cjs');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var Effect__namespace = /*#__PURE__*/_interopNamespace(Effect);
var LoggerUtils__namespace = /*#__PURE__*/_interopNamespace(LoggerUtils);
var LogLevel__namespace = /*#__PURE__*/_interopNamespace(LogLevel);

const minify = (operation) => Effect__namespace.succeed(
  operation.replace(/\s*#.*$/gm, "").replace(/\s+/gm, " ").trim()
);
const extractName = (operation) => Effect__namespace.succeed(operation).pipe(
  Effect__namespace.filterOrFail(
    (operation2) => {
      const match = operation2.match(/\b(query|mutation)\b\s+(\w+)/);
      return !!match;
    },
    (operation2) => new errors_js.ExtractOperationNameError(operation2)
  ),
  Effect__namespace.andThen((operation2) => {
    const match = operation2.match(/\b(query|mutation)\b\s+(\w+)/);
    return Effect__namespace.succeed(match[2]);
  })
);
const assert = ({
  type,
  operation
}) => Effect__namespace.succeed(operation).pipe(
  Effect__namespace.filterOrDie(
    (o) => type === "query" ? predicates_js.isQuery(o) : predicates_js.isMutation(o),
    (o) => type === "query" ? new errors_js.AssertQueryError(o) : new errors_js.AssertMutationError(o)
  )
);
const annotate = ({
  type,
  operation: o,
  variables
}) => LoggerUtils__namespace.filterLevelOrNever(
  LogLevel__namespace.None,
  Effect__namespace.gen(function* () {
    const name = yield* extractName(o);
    return yield* Effect__namespace.annotateLogsScoped({
      name,
      type,
      operation: o,
      variables
    });
  })
);
const validate = ({
  type,
  operation,
  variables
}) => Effect__namespace.gen(function* () {
  const minifiedOperation = yield* minify(operation);
  yield* assert({ type, operation: minifiedOperation });
  yield* annotate({
    type,
    operation: minifiedOperation,
    variables
  });
  return minifiedOperation;
});

exports.annotate = annotate;
exports.assert = assert;
exports.extractName = extractName;
exports.minify = minify;
exports.validate = validate;
//# sourceMappingURL=StorefrontOperation.cjs.map
//# sourceMappingURL=StorefrontOperation.cjs.map