'use strict';

var Effect = require('effect/Effect');
var StorefrontOperation = require('./StorefrontOperation.cjs');
var StorefrontClient = require('./StorefrontClient.cjs');
var LoggerUtils = require('./LoggerUtils.cjs');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var Effect__namespace = /*#__PURE__*/_interopNamespace(Effect);
var StorefrontOperation__namespace = /*#__PURE__*/_interopNamespace(StorefrontOperation);
var StorefrontClient__namespace = /*#__PURE__*/_interopNamespace(StorefrontClient);
var LoggerUtils__namespace = /*#__PURE__*/_interopNamespace(LoggerUtils);

const make = (options) => {
  const clientEffect = Effect__namespace.gen(function* () {
    const createClient = yield* StorefrontClient__namespace.StorefrontClient;
    const client = yield* createClient(options);
    const executeOperation = (type, originalOperation, options2) => Effect__namespace.gen(function* () {
      const operation = yield* StorefrontOperation__namespace.validate({
        type,
        operation: originalOperation,
        variables: Reflect.get(options2 || {}, "variables")
      });
      const response = yield* client.request(operation, options2);
      if (response?.errors) {
        yield* Effect__namespace.logError(response);
      } else {
        yield* Effect__namespace.logInfo(response);
      }
      return response;
    }).pipe(Effect__namespace.scoped);
    const query = (query2, options2) => executeOperation(
      "query",
      query2,
      options2
    ).pipe(LoggerUtils__namespace.withNamespacedLogSpan("Query"));
    const mutate = (mutation, options2) => executeOperation(
      "mutate",
      mutation,
      options2
    ).pipe(LoggerUtils__namespace.withNamespacedLogSpan("Mutation"));
    return {
      query,
      mutate
    };
  }).pipe(LoggerUtils__namespace.withNamespacedLogSpan("Create"));
  const cachedEffect = Effect__namespace.gen(function* () {
    const cachedClient = yield* Effect__namespace.cached(clientEffect);
    return yield* cachedClient;
  });
  return cachedEffect.pipe(
    Effect__namespace.catchTag("ParseError", (e) => Effect__namespace.dieMessage(e.message))
  );
};
const Default = StorefrontClient__namespace.Default;

exports.Default = Default;
exports.make = make;
//# sourceMappingURL=TypedStorefrontClient.cjs.map
//# sourceMappingURL=TypedStorefrontClient.cjs.map