import * as Data from 'effect/Data';

class BadRequestStatusError extends Data.TaggedError(
  "BadRequestStatusError"
) {
  static status = 400;
  constructor() {
    super({
      message: "The server will not process the request.",
      status: BadRequestStatusError.status
    });
  }
}
class PaymentRequiredStatusError extends Data.TaggedError(
  "PaymentRequiredStatusError"
) {
  static status = 402;
  constructor() {
    super({
      message: "The shop is frozen. The shop owner will need to pay the outstanding balance to unfreeze the shop.",
      status: PaymentRequiredStatusError.status
    });
  }
}
class ForbiddenStatusError extends Data.TaggedError(
  "ForbiddenStatusError"
) {
  static status = 403;
  constructor() {
    super({
      message: "The shop is forbidden. Returned if the store has been marked as fraudulent.",
      status: ForbiddenStatusError.status
    });
  }
}
class NotFoundStatusError extends Data.TaggedError(
  "NotFoundStatusError"
) {
  static status = 404;
  constructor() {
    super({
      message: "The resource isn\u2019t available. This is often caused by querying for something that\u2019s been deleted.",
      status: NotFoundStatusError.status
    });
  }
}
class LockedStatusError extends Data.TaggedError("LockedStatusError") {
  static status = 423;
  constructor() {
    super({
      status: LockedStatusError.status,
      message: "The shop isn\u2019t available. This can happen when stores repeatedly exceed API rate limits or due to fraud risk."
    });
  }
}
class RetriableStatusCodesError extends Data.TaggedError(
  "RetriableStatusCodesError"
) {
  static validStatuses = [429, 503];
  constructor(status) {
    super({
      message: "Something went wrong with the request. Attempted to retry the request, but failed.",
      status
    });
  }
}
class StorefrontServerStatusError extends Data.TaggedError(
  "StorefrontServerStatusError"
) {
  constructor(status) {
    super({
      message: "An internal error occurred in Shopify. Check out the Shopify status page for more information.",
      status
    });
  }
}
class AssertQueryError extends Data.TaggedError("AssertQueryError") {
  constructor(query) {
    super({
      message: "Invalid query string provided!",
      query
    });
  }
}
class AssertMutationError extends Data.TaggedError(
  "AssertMutationError"
) {
  constructor(mutation) {
    super({
      message: "Invalid mutation string provided!",
      mutation
    });
  }
}
class ExtractOperationNameError extends Data.TaggedError(
  "ExtractOperationNameError"
) {
  constructor(operation) {
    super({
      message: `Failed to extract operation name from operation: ${operation}`
    });
  }
}

export { AssertMutationError, AssertQueryError, BadRequestStatusError, ExtractOperationNameError, ForbiddenStatusError, LockedStatusError, NotFoundStatusError, PaymentRequiredStatusError, RetriableStatusCodesError, StorefrontServerStatusError };
//# sourceMappingURL=errors.js.map
//# sourceMappingURL=errors.js.map