import * as Effect from 'effect/Effect';
import * as StorefrontOperation from './StorefrontOperation.js';
import * as StorefrontClient from './StorefrontClient.js';
import * as LoggerUtils from './LoggerUtils.js';

const make = (options) => {
  const clientEffect = Effect.gen(function* () {
    const createClient = yield* StorefrontClient.StorefrontClient;
    const client = yield* createClient(options);
    const executeOperation = (type, originalOperation, options2) => Effect.gen(function* () {
      const operation = yield* StorefrontOperation.validate({
        type,
        operation: originalOperation,
        variables: Reflect.get(options2 || {}, "variables")
      });
      const response = yield* client.request(operation, options2);
      if (response?.errors) {
        yield* Effect.logError(response);
      } else {
        yield* Effect.logInfo(response);
      }
      return response;
    }).pipe(Effect.scoped);
    const query = (query2, options2) => executeOperation(
      "query",
      query2,
      options2
    ).pipe(LoggerUtils.withNamespacedLogSpan("Query"));
    const mutate = (mutation, options2) => executeOperation(
      "mutate",
      mutation,
      options2
    ).pipe(LoggerUtils.withNamespacedLogSpan("Mutation"));
    return {
      query,
      mutate
    };
  }).pipe(LoggerUtils.withNamespacedLogSpan("Create"));
  const cachedEffect = Effect.gen(function* () {
    const cachedClient = yield* Effect.cached(clientEffect);
    return yield* cachedClient;
  });
  return cachedEffect.pipe(
    Effect.catchTag("ParseError", (e) => Effect.dieMessage(e.message))
  );
};
const Default = StorefrontClient.Default;

export { Default, make };
//# sourceMappingURL=TypedStorefrontClient.js.map
//# sourceMappingURL=TypedStorefrontClient.js.map